from collective.techevent import _
from collective.techevent.fields.datagrid import DataGrid
from collective.z3cform.datagridfield.row import DictRow
from datetime import datetime
from plone.app.event.base import default_timezone
from plone.app.event.dx.behaviors import StartBeforeEnd
from plone.app.z3cform.widgets.datetime import DatetimeFieldWidget
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import DexterityContent
from plone.supermodel import model
from plone.supermodel.model import Schema
from zope import schema
from zope.interface import implementer
from zope.interface import invariant
from zope.interface import provider


class ITrack(Schema):
    """A track in the event."""

    id = schema.ASCIILine(
        title=_("Code"),
        description=_("Code of this track"),
        required=True,
    )
    title = schema.TextLine(
        title=_("Track"),
        description=_("Title of this track"),
        required=True,
    )
    color = schema.TextLine(
        title=_("Color"),
        description=_("Color used for this item"),
        default="#000000",
        required=True,
    )
    directives.widget(
        "color",
        frontendOptions={
            "widget": "colorPicker",
        },
    )


class ILevel(Schema):
    """A session level in the event."""

    id = schema.ASCIILine(
        title=_("Code"),
        description=_("Code of this level"),
        required=True,
    )
    title = schema.TextLine(
        title=_("Level"),
        description=_("Title of this level"),
        required=True,
    )
    color = schema.TextLine(
        title=_("Color"),
        description=_("Color used for this item"),
        default="#000000",
        required=True,
    )
    directives.widget(
        "color",
        frontendOptions={
            "widget": "colorPicker",
        },
    )


class IAudienceGroup(Schema):
    """A target audience in the event."""

    id = schema.ASCIILine(
        title=_("Code"),
        description=_("Code of this audience"),
        required=True,
    )
    title = schema.TextLine(
        title=_("Audience"),
        description=_("Title of this audience"),
        required=True,
    )
    color = schema.TextLine(
        title=_("Color"),
        description=_("Color used for this item"),
        default="#000000",
        required=True,
    )
    directives.widget(
        "color",
        frontendOptions={
            "widget": "colorPicker",
        },
    )


class IActivityDuration(Schema):
    """Duration, in minutes of an activity."""

    id = schema.ASCIILine(
        title=_("Code"),
        description=_("Code of this duration"),
        required=True,
    )
    title = schema.TextLine(
        title=_("Title"),
        description=_("Title of this duration"),
        required=True,
    )
    duration = schema.Int(
        title=_("Activity duration"),
        description=_("How long, in minutes, this activity will use."),
        default=60,
        required=True,
    )
    color = schema.TextLine(
        title=_("Color"),
        description=_("Color used for this item"),
        default="#000000",
        required=True,
    )
    directives.widget(
        "color",
        frontendOptions={
            "widget": "colorPicker",
        },
    )


@provider(IFormFieldProvider)
class ISettings(model.Schema):
    """Tech Event configuration."""

    model.fieldset(
        "event-settings",
        label=_("Event Settings"),
        fields=[
            "start",
            "end",
            "tracks",
            "levels",
            "audience",
            "durations_keynote",
            "durations_talk",
            "durations_training",
        ],
    )

    start = schema.Datetime(title=_("Start time"), required=False)
    directives.widget(
        "start",
        DatetimeFieldWidget,
        default_timezone=default_timezone,
        klass="event_start",
    )

    end = schema.Datetime(title=_("End time"), required=False)
    directives.widget(
        "end", DatetimeFieldWidget, default_timezone=default_timezone, klass="event_end"
    )

    @invariant
    def validate_start_end(data):
        if data.start and data.end and data.start > data.end:
            raise StartBeforeEnd(_("End time must be after start time."))

    tracks = DataGrid(
        title=_("Tracks"),
        description=_("All tracks available at this event."),
        required=False,
        value_type=DictRow(schema=ITrack),
        default=[],
    )
    directives.widget(
        "tracks",
        frontendOptions={
            "widget": "dataGridList",
            "widgetProps": {
                "allow_reorder": True,
            },
        },
    )

    levels = DataGrid(
        title=_("Session Levels"),
        description=_("Levels for a session."),
        required=False,
        value_type=DictRow(schema=ILevel),
        default=[],
    )
    directives.widget(
        "levels",
        frontendOptions={
            "widget": "dataGridList",
            "widgetProps": {
                "allow_reorder": True,
            },
        },
    )

    audience = DataGrid(
        title=_("Session Audience"),
        description=_("Target group for this audience."),
        required=False,
        value_type=DictRow(schema=ILevel),
        default=[],
    )
    directives.widget(
        "audience",
        frontendOptions={
            "widget": "dataGridList",
            "widgetProps": {
                "allow_reorder": True,
            },
        },
    )

    durations_keynote = DataGrid(
        title=_("Keynote: Slot duration"),
        description=_("Possible slot duration for a keynote"),
        required=False,
        value_type=DictRow(schema=IActivityDuration),
        default=[],
    )
    directives.widget(
        "durations_keynote",
        frontendOptions={
            "widget": "dataGridList",
            "widgetProps": {
                "allow_reorder": True,
            },
        },
    )

    durations_talk = DataGrid(
        title=_("Talk: Slot duration"),
        description=_("Possible slot duration for a talk"),
        required=False,
        value_type=DictRow(schema=IActivityDuration),
        default=[],
    )
    directives.widget(
        "durations_talk",
        frontendOptions={
            "widget": "dataGridList",
            "widgetProps": {
                "allow_reorder": True,
            },
        },
    )

    durations_training = DataGrid(
        title=_("Training: Slot duration"),
        description=_("Possible slot duration for a training"),
        required=False,
        value_type=DictRow(schema=IActivityDuration),
        default=[],
    )
    directives.widget(
        "durations_training",
        frontendOptions={
            "widget": "dataGridList",
            "widgetProps": {
                "allow_reorder": True,
            },
        },
    )


@implementer(ISettings)
class Settings:
    """Tech Event settings."""

    def __init__(self, context: DexterityContent):
        self.context = context

    @property
    def start(self) -> datetime:
        """Getter, read from context and return back"""
        return self.context.start

    @start.setter
    def start(self, value: datetime):
        """Setter, called by the form, set on context."""
        self.context.start = value

    @property
    def end(self) -> datetime:
        """Getter, read from context and return back"""
        return self.context.end

    @end.setter
    def end(self, value: datetime):
        """Setter, called by the form, set on context."""
        self.context.end = value

    @property
    def tracks(self) -> list[ITrack]:
        """Getter, read from context and return back"""
        return self.context.tracks

    @tracks.setter
    def tracks(self, value: list[ITrack]):
        """Setter, called by the form, set on context."""
        self.context.tracks = value

    @property
    def levels(self) -> list[ILevel]:
        """Getter, read from context and return back"""
        return self.context.levels

    @levels.setter
    def levels(self, value: list[ILevel]):
        """Setter, called by the form, set on context."""
        self.context.levels = value

    @property
    def audience(self) -> list[IAudienceGroup]:
        """Getter, read from context and return back"""
        return self.context.audience

    @audience.setter
    def audience(self, value: list[IAudienceGroup]):
        """Setter, called by the form, set on context."""
        self.context.audience = value

    @property
    def durations_keynote(self) -> list[IActivityDuration]:
        """Getter, read from context and return back"""
        return self.context.durations_keynote

    @durations_keynote.setter
    def durations_keynote(self, value: list[IActivityDuration]):
        """Setter, called by the form, set on context."""
        self.context.durations_keynote = value

    @property
    def durations_talk(self) -> list[IActivityDuration]:
        """Getter, read from context and return back"""
        return self.context.durations_talk

    @durations_talk.setter
    def durations_talk(self, value: list[IAudienceGroup]):
        """Setter, called by the form, set on context."""
        self.context.durations_talk = value

    @property
    def durations_training(self) -> list[IActivityDuration]:
        """Getter, read from context and return back"""
        return self.context.durations_training

    @durations_training.setter
    def durations_training(self, value: list[IAudienceGroup]):
        """Setter, called by the form, set on context."""
        self.context.durations_training = value
